import { Box } from '../../classes/display/box.js';
import { Scene } from '../../classes/display/scene.js';
import { Expert } from '../../classes/actors/expert.js';
import { PostContent } from '../../classes/supporting/post-content.js';
import { DAO } from '../../classes/dao/dao.js';
import { delayOrWait } from '../../classes/display/scene-controls.js';

export class ForumTest {
  constructor(options) {
    this.scene = null;
    this.dao = null;
    this.experts = null;
    this.posts = null;
    this.options = {
      defaultDelayMs: 1,
      poolDurationMs: 50,
      ...options,
    };
  }

  async addPost(authors, fee, citations = []) {
    const postIndex = this.posts.length;
    const title = `posts[${postIndex}]`;
    await this.scene.sequence.startSection();

    const postContent = new PostContent().setTitle(title);

    const submitter = Array.isArray(authors) ? authors[0].author : authors;

    if (Array.isArray(authors)) {
      for (const { author, weight } of authors) {
        console.log('author', { author, weight });
        postContent.addAuthor(author.reputationPublicKey, weight);
      }
    }

    for (const { postId, weight } of citations) {
      postContent.addCitation(postId, weight);
    }

    const { pool, postId } = await submitter.submitPostWithFee(
      postContent,
      {
        fee,
      },
      {
        duration: this.options.poolDurationMs,
        tokenLossRatio: 1,
      },
    );
    this.posts.push(postId);
    await delayOrWait(this.options.poolDurationMs);
    await pool.evaluateWinningConditions();
    await this.scene.sequence.endSection();
    await delayOrWait(this.options.defaultDelayMs);
    return postId;
  }

  async newExpert() {
    // Hide by default, for simplicity of rendering first 9 forum tests
    const options = {
      hide: !this.options.displayAuthors,
      announce: this.options.displayAuthors,
    };
    const index = this.experts.length;
    const name = `Expert${index + 1}`;
    const expert = await new Expert(this.dao, name, this.scene, options).initialize();
    this.experts.push(expert);
    return expert;
  }

  async setup() {
    const rootElement = document.getElementById('scene');
    const rootBox = new Box('rootBox', rootElement).flex();

    const scene = this.scene = new Scene('Forum test', rootBox);
    scene.withSequenceDiagram();
    scene.withFlowchart();
    scene.withTable();

    // If we're going to announce experts, announce the DAO so it appears first.
    this.dao = new DAO('DAO', scene, { announce: this.options.displayAuthors });
    this.forum = this.dao.forum;
    this.experts = [];
    this.posts = [];

    await this.newExpert();

    await this.dao.addComputedValue('total value', () => this.dao.reputation.getTotal());
  }
}
